'use strict';

import gulp     from 'gulp';
import plugins  from 'gulp-load-plugins';
import browser  from 'browser-sync';
import panini   from 'panini';
import rimraf   from 'rimraf';
import sherpa   from 'style-sherpa';
import named    from 'vinyl-named';
import webpack2 from 'webpack';
import webpackStream from 'webpack-stream';
import dartSass from 'sass';
import gulpSass from 'gulp-sass';

// Load all Gulp plugins into one variable
const $ = plugins();

// Load settings from config.js
const CONFIG = require('./config.js');

const sass = gulpSass(dartSass);

/// SASS ----------------------------------------------------------------------

// Compile Sass into CSS.
gulp.task('sass', () => {
  return gulp.src('src/scss/app.scss')
    .pipe($.sourcemaps.init())
    .pipe(sass({
      includePaths: CONFIG.SASS_INCLUDE_PATHS
    }).on('error', sass.logError))
    .pipe($.autoprefixer())
    .pipe($.sourcemaps.write('.'))
    .pipe(gulp.dest(CONFIG.BUILD_PATH + '/assets/css'))
    .pipe(browser.reload({ stream: true }));
});

// Compile Sass into CSS and compress it too.
gulp.task('sass:dist', gulp.series('sass', () => {
  console.log('Copying and compressing CSS...');

  return gulp.src(CONFIG.BUILD_PATH + '/assets/css/*')
    .pipe(gulp.dest(CONFIG.DIST_PATH + '/css'))
    .pipe($.filter([CONFIG.DIST_PATH + '/css/*.css']))
      .pipe($.cleanCss({ compatibility: 'ie9' }))
      .pipe($.rename({ suffix: '.min' }))
      .pipe(gulp.dest(CONFIG.DIST_PATH + '/css'));
}));

/// JavaScript ----------------------------------------------------------------

let webpackConfig = {
  devtool: 'source-map',
  module: {
    rules: [
      {
        test: /.js$/,
        use: [
          {
            loader: 'babel-loader'
          }
        ]
      }
    ]
  }
}

// Bundle JavaScript module.
gulp.task('javascript', () => {
  return gulp.src('src/js/app.js')
    .pipe(named())
    .pipe(webpackStream(webpackConfig, webpack2))
    .pipe(gulp.dest(CONFIG.BUILD_PATH + '/assets/js'));
});

// Bundle JavaScript module and compress it too.
gulp.task('javascript:dist', gulp.series('javascript', () => {
  console.log('Copying and compressing JavaScript...');

  return gulp.src(CONFIG.BUILD_PATH + '/assets/js/*')
    .pipe(gulp.dest(CONFIG.DIST_PATH + '/js'))
    .pipe($.filter([CONFIG.DIST_PATH + '/js/*.js']))
      .pipe($.uglify().on('error', e => { console.log(e); }))
      .pipe($.rename({ suffix: '.min' }))
      .pipe(gulp.dest(CONFIG.DIST_PATH + '/js'));
}));

/// Other assets --------------------------------------------------------------

// Copy images to the 'build' folder.
gulp.task('images', () => {
  return gulp.src('src/img/**/*')
    .pipe(gulp.dest(CONFIG.BUILD_PATH + '/assets/img'));
});

// Compress images to the 'dist' folder.
gulp.task('images:dist', () => {
  return gulp.src('src/img/**/*')
    .pipe($.imagemin({
      progressive: true
    }))
    .pipe(gulp.dest(CONFIG.DIST_PATH + '/img'));
});

// Copy other assets to the 'build' folder.
gulp.task('copy', () => {
  return gulp.src(CONFIG.ASSETS_FILES)
    .pipe(gulp.dest(CONFIG.BUILD_PATH + '/assets'));
});

// Copy other assets to the 'dist' folder.
gulp.task('copy:dist', () => {
  return gulp.src(CONFIG.ASSETS_FILES)
    .pipe(gulp.dest(CONFIG.DIST_PATH));
});

/// HTML files ----------------------------------------------------------------

// Generate HTML files from page templates.
gulp.task('mockup:pages', () => {
  return gulp.src('mockup/pages/**/*.{html,hbs,handlebars}')
    .pipe(panini({
      root: 'mockup/pages/',
      layouts: 'mockup/layouts/',
      partials: 'mockup/partials/',
      data: 'mockup/data/',
      helpers: 'mockup/helpers/'
    }))
    .pipe(gulp.dest(CONFIG.BUILD_PATH));
});

// Copy mockup's media to the 'build' folder.
gulp.task('mockup:media', () => {
  return gulp.src('mockup/media/**/*')
    .pipe(gulp.dest(CONFIG.BUILD_PATH + '/media'));
});

gulp.task('mockup', gulp.series('mockup:pages', 'mockup:media'));

// Generate a style guide from the Markdown content.
gulp.task('styleguide', done => {
  sherpa('styleguide/index.md', {
    output: CONFIG.BUILD_PATH + '/styleguide.html',
    template: 'styleguide/template.html'
  }, done);
});

/// General tasks -------------------------------------------------------------

// Build the assets and pages for development into the 'build' folder.
gulp.task('build',
  gulp.series(clean,
    gulp.parallel('mockup', 'sass', 'javascript', 'images', 'copy'),
    'styleguide'
));

// Build the compressed assets for production into the 'dist' folder.
gulp.task('build:dist',
  gulp.series(clean,
    gulp.parallel('sass:dist', 'javascript:dist', 'images:dist', 'copy:dist')
));

// Build the assets for production and watch for file changes.
gulp.task('watch:dist', gulp.series('build:dist', () => {
  gulp.watch(CONFIG.ASSETS_FILES)
    .on('all', gulp.series('copy:dist'));
  gulp.watch('src/scss/**/*.scss')
    .on('all', gulp.series('sass:dist'));
  gulp.watch('src/js/**/*.js')
    .on('all', gulp.series('javascript:dist'));
  gulp.watch('src/img/**/*')
    .on('all', gulp.series('images:dist'));
}));

// Run a development server and watch for file changes.
gulp.task('server',
  gulp.series('build', server, watch));

// Run a proxy server to the application and watch for file changes.
gulp.task('proxy',
  gulp.series('build:dist', proxyServer, 'watch:dist'));

// An alias to the 'build' task.
gulp.task('default',
  gulp.parallel('build'));

/// Internal tasks ------------------------------------------------------------

// Delete the "build" folder.
function clean(done) {
  rimraf(CONFIG.BUILD_PATH, done);
}

// Load updated HTML templates and partials into Panini.
function resetPages(done) {
  panini.refresh();
  done();
}

// Start a server with BrowserSync to preview the site in.
function server(done) {
  browser.init({
    server: CONFIG.BUILD_PATH,
    port: CONFIG.SERVER_PORT,
    notify: false,
    online: false
  });
  done();
}

// Start a proxy server with BrowserSync to preview the production site in.
function proxyServer(done) {
  browser.init({
    port: CONFIG.SERVER_PORT,
    proxy: CONFIG.SERVER_PROXY,
    notify: false,
    online: false
  });
  done();
}

// Watch for changes to static assets, pages, Sass, and JavaScript.
function watch() {
  gulp.watch(CONFIG.ASSETS_FILES)
    .on('all', gulp.series('copy'));

  gulp.watch('src/scss/**/*.scss')
    .on('all', gulp.series('sass'));
  gulp.watch('src/js/**/*.js')
    .on('all', gulp.series('javascript', browser.reload));
  gulp.watch('src/img/**/*')
    .on('all', gulp.series('images', browser.reload));

  gulp.watch('mockup/pages/**/*.html')
    .on('all', gulp.series('mockup:pages', browser.reload));
  gulp.watch('mockup/{layouts,partials}/**/*.html')
    .on('all', gulp.series(resetPages, 'mockup:pages', browser.reload));
  gulp.watch('mockup/data/**/*.{js,json,yml}')
    .on('all', gulp.series(resetPages, 'mockup:pages', browser.reload));
  gulp.watch('mockup/helpers/**/*.js')
    .on('all', gulp.series(resetPages, 'mockup:pages', browser.reload));
  gulp.watch('mockup/media/**/*')
    .on('all', gulp.series('mockup:media', browser.reload));

  gulp.watch('styleguide/**')
    .on('all', gulp.series('styleguide', browser.reload));
}
